//////////////////////////////////////////////////////////////////////////////////////////
// NcOled.c
//
// ■更新履歴
// 2011/06/25 NanaCraft
//            ・新規作成。
//
//////////////////////////////////////////////////////////////////////////////////////////

#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include "spi.h"
#include "oled.h"

extern volatile uint32_t gOELD_Orientation_Mode;

#define WRITE_RAM_BUFFER    (8)

///////////////////////////////////////////////////////////////////////////////
// OLED関連

void NcOled$SetColumnAddress(uint32_t startAddress, uint32_t endAddress)
{
    uint32_t oled[3];

    oled[0] = 0x15;
    oled[1] = (startAddress & 127) | 0x100;
    oled[2] = (endAddress   & 127) | 0x100;

    SPI_BulkTxData(oled, 3);
}

void NcOled$SetRowAddress(uint32_t startAddress, uint32_t endAddress)
{
    uint32_t oled[3];

    oled[0] = 0x75;
    oled[1] = (startAddress & 127) | 0x100;
    oled[2] = (endAddress   & 127) | 0x100;

    SPI_BulkTxData(oled, 3);
}

void NcOled$WriteRamBegin()
{
    SPI_TxData(0x5c);
}

void NcOled$WriteRam8(const uint8_t* data, uint32_t length)
{
    uint32_t dataList[WRITE_RAM_BUFFER];
    uint32_t i;
    uint32_t j;
    uint32_t iNumber;
    uint32_t remainder;
    uint32_t dataOffset;

    iNumber = length / WRITE_RAM_BUFFER;
    for (i = 0; i < iNumber; i++)
    {
        dataOffset = i * WRITE_RAM_BUFFER;
        for (j = 0; j < WRITE_RAM_BUFFER; j++)
        {
            dataList[j] = data[dataOffset + j] | 0x100;
        }

        SPI_BulkTxData(dataList, WRITE_RAM_BUFFER);
    }

    remainder = length % WRITE_RAM_BUFFER;
    if (remainder >= 1)
    {
        dataOffset = length - remainder;
        for (i = 0; i < remainder; i++)
        {
            dataList[i] = data[dataOffset + i] | 0x100;
        }

        SPI_BulkTxData(dataList, remainder);
    }
}

void NcOled$WriteRam16(const uint16_t* data, uint32_t length)
{
    uint16_t dataItem;
    uint32_t dataList[WRITE_RAM_BUFFER];
    uint32_t i;
    uint32_t j;
    uint32_t iNumber;
    uint32_t remainder;

    iNumber = length / (WRITE_RAM_BUFFER / 2);
    for (i = 0; i < iNumber; i++)
    {
        for (j = 0; j < WRITE_RAM_BUFFER / 2; j++)
        {
            dataItem = data[i * (WRITE_RAM_BUFFER / 2) + j];
            dataList[j * 2    ] = ((dataItem >> 8) & 0xff) | 0x100;
            dataList[j * 2 + 1] = ( dataItem       & 0xff) | 0x100;
        }

        SPI_BulkTxData(dataList, WRITE_RAM_BUFFER);
    }

    remainder = length % (WRITE_RAM_BUFFER / 2);
    if (remainder >= 1)
    {
        for (i = 0; i < remainder; i++)
        {
            dataItem = data[length - remainder + i];
            dataList[i * 2    ] = ((dataItem >> 8) & 0xff) | 0x100;
            dataList[i * 2 + 1] = ( dataItem       & 0xff) | 0x100;
        }

        SPI_BulkTxData(dataList, remainder);
    }
}

void NcOledFillBegin(uint32_t left, uint32_t top, uint32_t width, uint32_t height)
{
    switch (gOELD_Orientation_Mode)
    {
        case OLED_TOP_N:
        case OLED_TOP_S:
            NcOled$SetColumnAddress(left, left + width - 1);
            NcOled$SetRowAddress(top, top + height - 1);
            break;
        case OLED_TOP_W:
        case OLED_TOP_E:
            NcOled$SetRowAddress(left, left + width - 1);
            NcOled$SetColumnAddress(top, top + height - 1);
            break;
    }

    NcOled$WriteRamBegin();
}

void NcOledFill8(const uint8_t* data, uint32_t length)
{
    NcOled$WriteRam8(data, length);
}

void NcOledFill16(const uint16_t* data, uint32_t length)
{
    NcOled$WriteRam16(data, length);
}

//
///////////////////////////////////////////////////////////////////////////////
